@@ -3,21 +3,35 @@
 # Licence: Artistic 2.0
 
 
-summary.felm <- function(object,...) {
+summary.felm <- function(object,...,robust=FALSE) {
   z <- object
   res <- list()
   res$p <- z$p
-#  if(res$p == 0) {
-#    return()
-#  }
+  if(res$p == 0) {
+    res <- list(residuals=as.vector(z$full.residuals),p=0,call=z$call)
+    class(res) <- 'summary.felm'
+    return(res)
+  }
   res$terms <- z$terms
   res$call <- z$call
+  if(is.logical(robust) && robust) {
+    if(!is.null(object$cse)) {
+      coefficients <- cbind(z$coefficients,z$cse,z$ctval,z$cpval)
+      sdnam <- 'Cluster s.e.'
+    } else {
+      coefficients <- cbind(z$coefficients,z$rse,z$rtval,z$rpval)
+      sdnam <- 'Robust s.e'
+    }
+  } else {
+    sdnam <- 'Std. Error'
+    coefficients <- cbind(z$coefficients,z$se,z$tval,z$pval)
+  }
+  if(!is.null(coefficients)) {
+    dimnames(coefficients) <- 
+       list(names(z$coefficients),
+           c('Estimate',sdnam,'t value','Pr(>|t|)'))
 
-  coefficients <- cbind(z$coefficients,z$se,z$tval,z$pval)
-  if(!is.null(coefficients))
-    dimnames(coefficients) <- list(names(z$coefficients),
-                                     c('Estimate','Std. Error','t value','Pr(>|t|)'))
-
+  }
   res$coefficients <- coefficients
   res$residuals <- as.vector(z$full.residuals)
 
@@ -31,14 +45,20 @@ summary.felm <- function(object,...) {
   # fstat, p-value
 
   p <- z$p
-  rdf <- z$N - p
+
+# should we subtract 1 for the intercept?
+# a similar adjustment is done in summary.felm when computing p
+
+  rdf <- z$N - p - 1
   rss <- sum(z$full.residuals^2)
-  f <- z$fitted
+
   resvar <- rss/rdf
   sigma <- sqrt(resvar)
-#  mss <- sum( (f - mean(f))^2)
-  mss <- sum(f^2)
-  r2 <- mss/(mss+rss)
+
+  tss <- sum( (z$response - mean(z$response))^2)
+  mss <- tss - rss
+
+  r2 <- mss/tss
   r2adj <- 1-(1-r2)*(z$N/rdf)
   fstat <- (mss/p)/resvar
 
@@ -68,7 +88,7 @@ print.summary.felm <- function(x,digits=max(3,getOption('digits')-3),...) {
   print(qres,digits=digits)
 
   cat('\nCoefficients:\n')
-  if(x$p == 0)
+  if(x$p <= 0)
     cat("(No coefficients)\n")
   else {
     printCoefmat(x$coefficients,digits=digits)
